aboutsummaryrefslogtreecommitdiffstats
path: root/netlib/tcp.py
diff options
context:
space:
mode:
Diffstat (limited to 'netlib/tcp.py')
-rw-r--r--netlib/tcp.py112
1 files changed, 46 insertions, 66 deletions
diff --git a/netlib/tcp.py b/netlib/tcp.py
index 33f7ef3a..e48f4f6b 100644
--- a/netlib/tcp.py
+++ b/netlib/tcp.py
@@ -173,7 +173,40 @@ class Reader(_FileLike):
return result
-class TCPClient:
+class SocketCloseMixin:
+ def finish(self):
+ self.finished = True
+ try:
+ if not getattr(self.wfile, "closed", False):
+ self.wfile.flush()
+ self.close()
+ self.wfile.close()
+ self.rfile.close()
+ except (socket.error, NetLibDisconnect):
+ # Remote has disconnected
+ pass
+
+ def close(self):
+ """
+ Does a hard close of the socket, i.e. a shutdown, followed by a close.
+ """
+ try:
+ if self.ssl_established:
+ self.connection.shutdown()
+ self.connection.sock_shutdown(socket.SHUT_WR)
+ else:
+ self.connection.shutdown(socket.SHUT_WR)
+ #Section 4.2.2.13 of RFC 1122 tells us that a close() with any pending readable data could lead to an immediate RST being sent.
+ #http://ia600609.us.archive.org/22/items/TheUltimateSo_lingerPageOrWhyIsMyTcpNotReliable/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable.html
+ while self.connection.recv(4096):
+ pass
+ self.connection.close()
+ except (socket.error, SSL.Error, IOError):
+ # Socket probably already closed
+ pass
+
+
+class TCPClient(SocketCloseMixin):
rbufsize = -1
wbufsize = -1
def __init__(self, host, port, source_address=None, use_ipv6=False):
@@ -228,43 +261,21 @@ class TCPClient:
def gettimeout(self):
return self.connection.gettimeout()
- def close(self):
- """
- Does a hard close of the socket, i.e. a shutdown, followed by a close.
- """
- try:
- if self.ssl_established:
- self.connection.shutdown()
- self.connection.sock_shutdown(socket.SHUT_WR)
- else:
- self.connection.shutdown(socket.SHUT_WR)
- #Section 4.2.2.13 of RFC 1122 tells us that a close() with any pending readable data could lead to an immediate RST being sent.
- #http://ia600609.us.archive.org/22/items/TheUltimateSo_lingerPageOrWhyIsMyTcpNotReliable/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable.html
- while self.connection.recv(4096):
- pass
- self.connection.close()
- except (socket.error, SSL.Error, IOError):
- # Socket probably already closed
- pass
-
-class BaseHandler:
+class BaseHandler(SocketCloseMixin):
"""
The instantiator is expected to call the handle() and finish() methods.
"""
rbufsize = -1
wbufsize = -1
- def __init__(self, connection, client_address, server):
+ def __init__(self, connection):
self.connection = connection
self.rfile = Reader(self.connection.makefile('rb', self.rbufsize))
self.wfile = Writer(self.connection.makefile('wb', self.wbufsize))
- self.client_address = client_address
- self.server = server
self.finished = False
self.ssl_established = False
-
self.clientcert = None
def convert_to_ssl(self, cert, key, method=SSLv23_METHOD, options=None, handle_sni=None, request_client_cert=False, cipher_list=None):
@@ -318,43 +329,12 @@ class BaseHandler:
self.rfile.set_descriptor(self.connection)
self.wfile.set_descriptor(self.connection)
- def finish(self):
- self.finished = True
- try:
- if not getattr(self.wfile, "closed", False):
- self.wfile.flush()
- self.close()
- self.wfile.close()
- self.rfile.close()
- except (socket.error, NetLibDisconnect):
- # Remote has disconnected
- pass
-
def handle(self): # pragma: no cover
raise NotImplementedError
def settimeout(self, n):
self.connection.settimeout(n)
- def close(self):
- """
- Does a hard close of the socket, i.e. a shutdown, followed by a close.
- """
- try:
- if self.ssl_established:
- self.connection.shutdown()
- self.connection.sock_shutdown(socket.SHUT_WR)
- else:
- self.connection.shutdown(socket.SHUT_WR)
- # Section 4.2.2.13 of RFC 1122 tells us that a close() with any
- # pending readable data could lead to an immediate RST being sent.
- # http://ia600609.us.archive.org/22/items/TheUltimateSo_lingerPageOrWhyIsMyTcpNotReliable/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable.html
- while self.connection.recv(4096):
- pass
- except (socket.error, SSL.Error):
- # Socket probably already closed
- pass
- self.connection.close()
class TCPServer:
@@ -371,13 +351,13 @@ class TCPServer:
self.port = self.server_address[1]
self.socket.listen(self.request_queue_size)
- def request_thread(self, request, client_address):
+ def connection_thread(self, connection, client_address):
try:
- self.handle_connection(request, client_address)
- request.close()
+ self.handle_client_connection(connection, client_address)
except:
- self.handle_error(request, client_address)
- request.close()
+ self.handle_error(connection, client_address)
+ finally:
+ connection.close()
def serve_forever(self, poll_interval=0.1):
self.__is_shut_down.clear()
@@ -391,10 +371,10 @@ class TCPServer:
else:
raise
if self.socket in r:
- request, client_address = self.socket.accept()
+ connection, client_address = self.socket.accept()
t = threading.Thread(
- target = self.request_thread,
- args = (request, client_address)
+ target = self.connection_thread,
+ args = (connection, client_address)
)
t.setDaemon(1)
t.start()
@@ -410,7 +390,7 @@ class TCPServer:
def handle_error(self, request, client_address, fp=sys.stderr):
"""
Called when handle_connection raises an exception.
"""
# If a thread has persisted after interpreter exit, the module might be
# none.
@@ -421,7 +401,7 @@ class TCPServer:
print >> fp, exc
print >> fp, '-'*40
- def handle_connection(self, request, client_address): # pragma: no cover
+ def handle_client_connection(self, conn, client_address): # pragma: no cover
"""
Called after client connection.
"""
@@ -410,7 +390,7 @@ class TCPServer:
def handle_error(self, request, client_address, fp=sys.stderr):
"""
- Called when handle_connection raises an exception.
+ Called when handle_client_connection raises an exception.
"""
# If a thread has persisted after interpreter exit, the module might be